#!/usr/bin/env Rscript

library(ape)
library(tidyverse)
requireNamespace("phangorn")
requireNamespace("apTreeshape")

options(mc.cores = parallel::detectCores())

fns <- Sys.glob("output/au_test/*.txt")

js <- jsonlite::fromJSON("https://fishtreeoflife.org/api/family.json")

sizes <- lapply(names(js), function(ff) {
    lf <- js[[ff]]
    data_frame(family = ff, richness = length(lf$species), ntip = length(lf$sampled_species), frac = ntip / richness, rogues = length(lf$rogues))
}) %>% bind_rows()


au_spec <- cols(
  .default = col_character(),
  rank = col_integer(),
  item = col_integer(),
  obs = col_double(),
  au = col_double(),
  np = col_double(),
  bp = col_double(),
  pp = col_double(),
  kh = col_double(),
  sh = col_double(),
  wkh = col_double(),
  wsh = col_double()
)

au_names <- c("x1", "rank", "item", "obs", "au", "au_se", "np", "np_se", "x2", "bp", "bp_se", "pp", "pp_se", "kh", "kh_se", "sh", "sh_se", "wkh", "wkh_se", "wsh", "wsh_se", "x3")


au_results <- parallel::mclapply(fns, function(fn) {
    fam <- basename(fn) %>% str_replace("\\.txt$", "")

    au_result <- read_table(fn, skip=3, col_names = au_names, col_types = au_spec) %>%
        select(-starts_with("x")) %>%
        filter(!is.na(rank)) %>%
        mutate_at(vars(ends_with("se")), funs(as.numeric(str_replace_all(., "[)(]", "")))) %>% 
        mutate(family = fam, item = ifelse(item == 1, "pruned", "reanalyzed"))
}) %>% bind_rows()

completeness <- parallel::mclapply(mc.preschedule = F, unique(au_results$family), function(ff) {
    dd <- as.character(read.dna(file.path("raxml", ff, "dat.phy")))
    dd <- dd[, !!colSums(dd == "-")] # trim all indetermined characters
    data_frame(family = ff, align_gaps = sum(dd == "-"), align_total = length(dd), align_length = length(dd[1, ]), align_incomplete = align_gaps / align_total)
}) %>% bind_rows()

distances <- parallel::mclapply(mc.preschedule = F, unique(au_results$family), function(ff) {
    pruned <- read.tree(file.path("output/pruned_trees/", paste0(ff, ".tre"))) %>% root(., Ntip(.), resolve.root = T)
    raxml <- read.tree(file.path("output/raxml_trees/", paste0(ff, ".tre"))) %>% root(., Ntip(.), resolve.root = T)
    rf <- phangorn::RF.dist(pruned, raxml, normalize = TRUE, check.labels = TRUE)
    shp_pruned <- apTreeshape::as.treeshape(pruned) %>% apTreeshape::colless(norm = "yule")
    shp_raxml <- apTreeshape::as.treeshape(raxml) %>% apTreeshape::colless(norm = "yule")
    data_frame(family = ff, rf_dist = rf, shape = shp_pruned - shp_raxml)
}) %>% bind_rows()

results <- left_join(au_results, sizes) %>% left_join(completeness) %>% left_join(distances)

filt <- filter(results, item == "pruned")

write_csv(filt, "au_results.csv")

fitted <- lm(au ~ align_incomplete * log(richness) * shape, data = filt)

simple_model <- lm(au ~ align_incomplete * log(richness), data = filt)
simple_with_frac <- lm(au ~ align_incomplete * log(richness) * frac, data = filt)
complex_model <- lm(au ~ align_incomplete * log(richness) * shape * rf_dist, data = filt)
anova(simple_model, simple_with_frac, complex_model)

gg <- expand.grid(align_incomplete = c(0.25, 0.5, 0.75), richness = c(10, 100, 1000), frac = c(0.25, 0.5, 0.75))
predicted <- gg %>% mutate(predicted_au = predict(simple_with_frac, gg), se_au = predict(simple_with_frac, gg, se = T)$se) %>% arrange(predicted_au)
#car::avPlots(fitted)

glm(au < 0.05 ~ align_incomplete * log(richness) * frac, data = filt)

lm(shape ~ align_incomplete * log(richness) * rf_dist, data = filt)

# false discovery rate p-value adjustment
pval <- 0.05
filt$au_adj <- p.adjust(filt$au, method = "fdr")

#ggplot(filt, aes(x = au)) + geom_dotplot(binwidth = 0.01, stackratio = 1.5) + geom_vline(xintercept = pval, lty = 2, col = "red")


pp <- ggplot(filt, aes(y = align_incomplete, x = au)) + geom_point(aes(size = frac, color = au_adj < pval), alpha = 0.75) + geom_text(aes(label = family), hjust = 0, nudge_x = 0.01, size = 3, data = filt) + scale_size_area(max_size = 4) + xlim(0, 0.9) + labs(size = "Sampling fraction", color = "Reject 'pruned' topology", x = "Approximately-unbiased test p-value", y = "Sequence incompleteness") + theme(legend.position = "top")
ggsave(pp, file = "au_test_mat_incomplete.png", width = 8, height = 4)
ggsave(pp, file = "au_test_mat_incomplete.pdf", width = 8, height = 4)

pp <- ggplot(filt, aes(y = richness, x = au)) + geom_point(aes(size = frac, color = au_adj < pval), alpha = 0.6) + geom_text(aes(label = family), hjust = 0, nudge_x = 0.01, size = 3, data = filt) + scale_size_area(max_size = 4) + xlim(0, 0.9) + scale_y_log10() + labs(size = "Sampling fraction", color = "Reject 'pruned' topology", x = "Approximately-unbiased test p-value", y = "Species richness") + theme(legend.position = "top")
pp <- ggplot(filt, aes(y = richness, x = au)) + geom_point(aes(size = frac, color = au_adj < pval), alpha = 0.6) + scale_size_area(max_size = 4) + xlim(0, 0.9) + scale_y_log10() + labs(size = "Sampling fraction", color = "Reject 'pruned' topology", x = "Approximately-unbiased test p-value", y = "Species richness") + scale_color_manual(values = c(`TRUE` = "#600000", `FALSE` = "grey75"), guide = FALSE) + theme_minimal() + theme(legend.position = "top")
ggsave(pp, file = "au_test_richness.png", width = 7, height = 4)
ggsave(pp, file = "au_test_richness.pdf", width = 7, height = 4)

