# writes a named list of dna matrices (from read.dna) to a raxml-style partitions file
write_partitioned_phylip <- function(partlist, outname) {
    phyname <- paste0(outname, ".phy")
    partname <- paste0(outname, ".partitions")

    all_together <- do.call(cbind, partlist)
    write.dna(all_together, phyname, format = "sequential", nbcol = -9, colsep = "")
    message("Wrote to ", phyname)

    sink(partname)
    idx <- 1
    for (ii in names(partlist)) {
        cat("DNA,", ii, "=", as.character(idx), "-", as.character(idx + ncol(partlist[[ii]]) - 1), fill = T)
        idx <- idx + ncol(partlist[[ii]])
    }
    sink(NULL)

    message("Wrote to ", partname)
}


# splits a dna matrix (from read.dna) into a named list of dna matrices based on a raxml-style partitions file
split_seqs <- function(sequence, raxml_partition_file) {
    partitions <- readLines(raxml_partition_file)
    splat <- partitions %>% str_replace_all("DNA, ", "") %>% str_split_fixed("[= -]+", 3)
    part_names <- splat[, 1]
    part_start <- as.integer(splat[, 2])
    part_end <- as.integer(splat[, 3])
    result = list()
    for (ii in 1:length(part_names)) {
        result[[part_names[ii]]] <- sequence[, part_start[ii]:part_end[ii]]
    }
    result
}

count_tax_with_data <- function(sequence) {
    sum(!apply(sequence, 1, function(x) all(x == as.raw(04))))
}

eol_digit <- function(x) { as.numeric(str_extract(x, "[0-9.]+$")) }

read_raxml_model_parameters <- function(infile) {
    lines <- readLines(infile)
    out <- list()
    patt <- list()
    tmp <- 0
    name <- ""
    cur <- list()
    for (line in lines) {
        if (str_detect(line, "^Alignment Patterns: ")) {
            tmp <- eol_digit(line)
        } else if (str_detect(line, "^Name: ")) {
            name <- str_extract(line, "\\b\\w+$")
            patt[[name]] <- tmp
        } else if (str_detect(line, "^Model Parameters of Partition ")) {
            name <- str_match(line, "Name: (\\w+),")[, 2]
        } else if (str_detect(line, "^alpha: ")) {
            cur$alpha <- eol_digit(line)
        } else if (str_detect(line, "^Tree-Length: ")) {
            cur$tl <- eol_digit(line)
        } else if (str_detect(line, "^rate A <-> C: ")) {
            cur$ac <- eol_digit(line)
        } else if (str_detect(line, "^rate A <-> G: ")) {
            cur$ag <- eol_digit(line)
        } else if (str_detect(line, "^rate A <-> T: ")) {
            cur$at <- eol_digit(line)
        } else if (str_detect(line, "^rate C <-> G: ")) {
            cur$cg <- eol_digit(line)
        } else if (str_detect(line, "^rate C <-> T: ")) {
            cur$ct <- eol_digit(line)
        } else if (str_detect(line, "^rate G <-> T: ")) {
            cur$gt <- eol_digit(line)
        } else if (str_detect(line, "^freq pi\\(A\\): ")) {
            cur$a <- eol_digit(line)
        } else if (str_detect(line, "^freq pi\\(C\\): ")) {
            cur$c <- eol_digit(line)
        } else if (str_detect(line, "^freq pi\\(G\\): ")) {
            cur$g <- eol_digit(line)
        } else if (str_detect(line, "^freq pi\\(T\\): ")) {
            cur$t <- eol_digit(line)
            cur$patterns <- patt[[name]]
            out[[name]] <- cur
            cur <- list()
            name <- ""
        }
    }
    reshape2::melt(out) %>% rename(gene = L1, variable = L2)
}
