#!/usr/bin/env Rscript

library(magrittr)
library(dplyr)
library(stringr)
library(jsonlite)
library(ape)

source("lib.R")

OUTPATH <- "raxml"
options(mc.cores = parallel::detectCores())

baseurl <- "https://fishtreeoflife.org/"
lf <- fromJSON(paste0(baseurl, "api/family.json"))

fn <- function(family) {
    if (!"matrix_phylip" %in% names(lf[[family]])) return()
    if (!"phylogram" %in% names(lf[[family]])) return()

    # sequences
    seqs_url <- paste0(baseurl, lf[[family]]$matrix_phylip)
    temp <- tempfile()
    download.file(seqs_url, temp)
    seqs <- read.dna(temp, format = "sequential")
    unlink(temp)

    # tree
    tree <- read.tree(paste0(baseurl, lf[[family]]$phylogram))

    # filter partitions out where fewer than 3 species have data
    partitioned_seqs <- split_seqs(seqs, paste0(baseurl, "downloads/final_alignment.partitions"))
    wanted <- Filter(function(x) count_tax_with_data(x) > 2, partitioned_seqs)

    # go to the next family if this means we don't have any more data
    if (length(wanted) == 0) return()

    # filter out taxa that are completely undetermined
    to_keep <- do.call(cbind, wanted) %>% apply(1, function(x) all(x == as.raw(04))) %>% not %>% which %>% names

    # go to the next family if this takes out too many species
    if (length(to_keep) <= 3) return()

    wanted <- lapply(wanted, function(x) x[to_keep, ])
    tree <- drop.tip(tree, tree$tip.label[!tree$tip.label %in% to_keep])

    dir.create(file.path(OUTPATH, family), recursive = TRUE, showWarnings = FALSE)
    write.tree(tree, file.path(OUTPATH, family, "dat.tre"))
    write_partitioned_phylip(wanted, file.path(OUTPATH, family, "dat"))
}

invisible(parallel::mclapply(names(lf), fn))
