#!/usr/bin/env Rscript

library(ape)
library(fishtree)
library(tidyverse)
library(ggtree)

# minimal code ripped from MonoPhy package
source("monophy_minimal.R")

au_results <- read_csv("au_results.csv")

get_species_for_taxa <- function(taxa) {
    fishtree_taxonomy(taxa)[[1]]$sampled_species
}

transformed_results <- au_results %>% select(family, au) %>% group_by(family, au) %>% do(species = get_species_for_taxa(.$family)) %>% unnest() %>% transmute(species = str_replace_all(species, " ", "_"), family)

tre <- fishtree_phylogeny()
tre <- drop.tip(tre, tre$tip.label[!tre$tip.label %in% transformed_results$species])

monophy_results <- AssessMonophyly(tre, as.data.frame(select(transformed_results, species, family)), verbosity = 0)

# Cleanse this land
evil <- c(unlist(monophy_results$label$IntruderTips, use.names = F), unlist(monophy_results$label$OutlierTips, use.names = F))
tre2 <- drop.tip(tre, evil)
tax2 <- filter(transformed_results, !species %in% evil)
mono <- monophy_results$label$result["Monophyly"]
mono$label <- rownames(monophy_results$label$result)

# wrapper for mrca that returns NA instead of null if the clade is monotypic
mrca_na <- function(phy, tip) {
    r <- getMRCA(phy, tip)
    if (is.null(r)) return(NA)
    return(r)
}

# compute mcra nodes and the ages of those
btimes <- branching.times(tre2)
stats <- tax2 %>% group_by(family) %>% summarise(node = mrca_na(tre2, species), depth = btimes[as.character(node)], N = n(), exemplar = first(species)) %>% left_join(select(au_results, family, au))

# drop the extra tips that we don't need
skeleton <- drop.tip(tre2, which(!tre2$tip.label %in% stats$exemplar))
skeleton$tip.label <- stats$family[match(skeleton$tip.label, stats$exemplar)]
skeleton <- ladderize(skeleton, right = FALSE)

gg <- ggtree(skeleton, layout = "fan", open.angle = 180, lwd = 0.25)

# truncate at 10 so we have room for tip labels
trunc <- -log(0.0001)
mult <- 5
gg$data <- gg$data %>% left_join(transmute(stats, label = family, richness = N, au)) %>% group_by(node) %>% mutate(au_trans = min(-log(au), trunc) * mult) %>% ungroup () %>% mutate(au_significant = p.adjust(au, method = "fdr") < 0.05)

gg2 <- gg +
    geom_segment(aes(x = x + 2, y = y, xend = x + au_trans, yend = y, colour = au_significant), data = filter(gg$data, isTip)) + scale_color_manual(values = c("grey50", "#e34a33"))
ggsave("au_backbone_plot.pdf", height = 8, width = 8)
system("pdfcrop au_backbone_plot.pdf au_backbone_plot.pdf")
system("convert -density 300 au_backbone_plot.pdf au_backbone_plot.png")

gg3 <- gg + geom_tiplab2(offset = trunc * mult, cex = 1) +
    geom_segment(aes(x = x + 2, y = y, xend = x + trunc * mult, yend = y), lwd = 0.1, lty = 2, color = "grey50", data = filter(gg$data, isTip)) +
    geom_segment(aes(x = x + 2, y = y, xend = x + au_trans, yend = y, colour = au_significant), data = filter(gg$data, isTip)) + scale_color_manual(values = c("grey50", "#e34a33"))

ggsave("au_backbone_plot_tiplab.pdf", height = 8, width = 8)
system("pdfcrop au_backbone_plot_tiplab.pdf au_backbone_plot_tiplab.pdf")
